--[[
	author: Zax
	note: 
	content:
		functions: SlashLib.
		globales: SLIB_
		XML: Cmdli_
		main frames: SlashLibFrameMain, SlashLibFrameEdit
		XML config: 
	todo:
		- modif ordre lignes dans liste edit (colonne 1) : en utilisant "index" à la place de sort alpha : voir .sortCmds() et .moveUpDown()
		- localisation FR
--]]

---------------------------------------------------------------------------------------
------------------------------------- VARIABLES ---------------------------------------
SlashLib = {} -- prefixe fonctions
local addonName = ...

local addonVar = {
	------------------------ nécessaire !!!
	["name"] = addonName, -- 
	["version"] = GetAddOnMetadata(addonName, "Version"), -- ne pas convertir en nombre ici !
	["chatColor"] = "92D26A",
	["slashCmd"] = "slashlib",
	["settingsPanel"] = "", -- nom du frame de settings : laisser vide si utilisation du frame de WoW (ou si pas de settings)
	["minimap"] = "", -- nécessite le fichier _minimap.lua (laisser vide ou NIL si pas de bouton minimap)
	------------------------ btn
	["btnWidth"] = 65, -- btn popup menu
	["colorGeneral"] = "6892F2",
	["colorPerChar"] = "35BFAE",
	------------------------ edit UI
	["widthUI"] = 760, -- px
	["widthList"] = 560, -- px (macros user)
	["alphaUI"] = 0.95,
	["margin"] = 4, -- px
	["maxVisibleLines"] = 11, -- nombre de lignes visibles dans les listes d'édition
	["vLine"] = 16, -- hauteur en pixel d'un bouton-ligne de la liste
	["memMaxLines"] = 50,-- lignes mémorisées pour chaque catégorie -- UNUSED
}

local cmdKind = { -- liste des genres d'actions (seul le type 2 - macro - est pris en compte dans la version 4.x)
	[1] = {
		["label"] = "SCRIPT",
		["bkcolor"] = "568BEF",
	},
	[2] = {
		["label"] = "MACRO",
		["bkcolor"] = "A75541",
	},
}

function SlashLib.getLocal(key) -- pour traduction (voir localization.lua) : ne pas deplacer !!!
	if (SLIB_LOCAL_TBL[key] == nil) then return key else return SLIB_LOCAL_TBL[key] end
end

-------------------------------------------- saved et init
SLIB_SAVED_DATA = {}
SLIB_SAVED_PERCHAR = {}
local slibSaved = {}
local slibSavedPerchar = {}
	
function SlashLib.initData(onlySettingsBoolean, subSettingsValue)
	local tbl = {
		["settings"] = {
			["version"] = addonVar["version"],
			["debug"] = 0,
		},
		["lines"] = {
			SlashLib.getNewMacro(1, 1, 2, false, "Addons Memory Usage", "/script print(gcinfo()..\" Kb of RAM are actually used by your addons.\")", true),
			SlashLib.getNewMacro(2, 2, 2, false, "SlashLib Help", "/slashlib", true),
			SlashLib.getNewMacro(3, 3, 2, false, "Macro Help", "/macrohelp", true),
		},
	}
	
	if (SlashLib.isTrue(onlySettingsBoolean)) then
		if (subSettingsValue ~= nil) then return tbl["settings"][subSettingsValue] else return tbl["settings"] end
	else
		return tbl
	end
end

function SlashLib.initDataPerChar(onlySettingsBoolean, subSettingsValue)
	local tbl = {
		["settings"] = {
			["visible"] = 1,
			["alpha"] = 0.8, -- btn alpha
			["posX"] = 200, -- btn pos X
			["posY"] = 600, -- btn pos Y : distance bas UIParent / haut main frame
		},
		["lines"] = {
			SlashLib.getNewMacro(1, 1, 2, true, "Abs", "/sit\n/abs", true),
		},
	}
	
	if (SlashLib.isTrue(onlySettingsBoolean)) then
		if (subSettingsValue ~= nil) then return tbl["settings"][subSettingsValue] else return tbl["settings"] end
	else
		return tbl
	end
end

function SlashLib.convertOldv3Cmds(oldSaved) -- fcn spéciale pour passer de la v3.x à la v4
--[[
SLIB_SAVED_DATA = { -- OLD v3.x
	{"Macro Binding","/script SetBindingMacro(\"KEY_NAME\",\"MACRO_NAME\")\n/script SaveBindings(2)-- 2 = only for this character",3},
	{"Sample Emote","/sit\n/abs",2},
	{"Sample Script","/script DEFAULT_CHAT_FRAME:AddMessage(gcinfo()..\" Kb of RAM are actually used by your addons.\",0,1,0)",3},
	{"Recount cfg","/recount config",1},
}
--]]

	local tblConvertedCmds = {}
	if (type(oldSaved) == "table") then
		for i, value in ipairs(oldSaved) do
			if ((not SlashLib.isEmpty(value[1])) and (not SlashLib.isEmpty(value[2]))) then
				table.insert(tblConvertedCmds, SlashLib.getNewMacro(i + 1000, i, 2, false, value[1], value[2], true))
			end
		end
	end
	return tblConvertedCmds
end

---------------------------------------------------------------------------------------
--------------------------------------- EVENTS ----------------------------------------
function SlashLib.eventHandler(self, event,...)
	if (event == "PLAYER_LOGIN") then
		SlashLib.loading()
	end
end

---------------------------------------------------------------------------------------
--------------------------------- CHARGEMENT & BINDINGS -------------------------------
function SlashLib.loading()
	-----------------------------
	--SlashLibFrameMain:RegisterEvent("PLAYER_LOGOUT")
	----------------------------- slash commandes
	SlashCmdList["SLIB_CMD"] = SlashLib.slashCmd
		SLASH_SLIB_CMD1 = "/"..addonVar["slashCmd"]
		SLASH_SLIB_CMD2 = "/slib"
				
	------------------------------- bindings
	--
	
	local tblSettings
	local updatedPrefs = false
	------------------------------- settings généraux
	if (SlashLib.isEmpty(SLIB_SAVED_DATA)) then
--print("chargement() : === no lua file : INIT SETTINGS ===")--/////
		SLIB_SAVED_DATA = SlashLib.initData(false)
--SlashLib.echoTable(SLIB_SAVED_DATA)--/////
	else
		if (SLIB_SAVED_DATA["lines"] == nil) then -- anciennes prefs v3.x
			local tbl = SlashLib.convertOldv3Cmds(SLIB_SAVED_DATA) -- MAJ en v4
			wipe(SLIB_SAVED_DATA)
			SLIB_SAVED_DATA = SlashLib.initData(false)
			SLIB_SAVED_DATA["lines"] = tbl
			updatedPrefs = true
		end
		tblSettings = SlashLib.initData(true) -- only settings
		if (SlashLib.tableKeyDiff(tblSettings, SLIB_SAVED_DATA["settings"], "") ~= nil) then
			SLIB_SAVED_DATA["settings"] = SlashLib.copyTable(tblSettings, false)
--print("chargement() : === diff saved data INIT SETTINGS ===")--/////
		end
	end
	------------------------------- settings perso
	if (SlashLib.isEmpty(SLIB_SAVED_PERCHAR)) then
		SLIB_SAVED_PERCHAR = SlashLib.initDataPerChar(false)
		SlashLib.testPrint("=== no perso lua file : INIT perso SETTINGS ===")
	else
		tblSettings = SlashLib.initDataPerChar(true) -- only settings
		if (SlashLib.tableKeyDiff(tblSettings, SLIB_SAVED_PERCHAR["settings"], "") ~= nil) then
			SLIB_SAVED_PERCHAR["settings"] = SlashLib.copyTable(tblSettings, false)
			SlashLib.testPrint("=== diff perso saved data : INIT perso SETTINGS ===")
		end
	end
	------------------------------- svgde
	slibSaved = SLIB_SAVED_DATA["lines"]
	slibSavedPerchar = SLIB_SAVED_PERCHAR["lines"]

	------------------------------- suite
	SlashLib.ui_createMainFrame()
	if (SlashLib.isTrue(SLIB_SAVED_PERCHAR["settings"]["visible"])) then
		SlashLib.showHide(true, false)
	end
	--------------- minimap (nécessite fichier _minimap.lua
	if (not SlashLib.isEmpty(addonVar["minimap"])) then
		--SlashLib.minimapCreate(addonVar["minimap"])
		--if (_G[addonVar["minimap"]] ~= nil) then SlashLib.minimapOnOff(SLIB_SAVED_DATA["settings"]["minimap"]["visible"]) end
	end
	SlashLib.createEditUI() -- fichier _edit
	--------------- loaded
	local str = ""
	--if (not SlashLibFrameMain:IsVisible() and not SlashLib.minimapIsVisible()) then str = " "..SlashLib.getLocal("Type /cmdline to display the console.") end
	if (not SlashLibFrameMain:IsVisible()) then str = " "..SlashLib.getLocal("Type /slashlib show to display the commands menu.") end
	if (SlashLib.debug()) then
		SetCVar("scriptErrors", 1)
		str = str..SlashLib.getTxtColor(" ------ DEBUG ON.", "red")
	end
	if (str ~= "") then SlashLib.echoChatAppl(str, true, nil) end
	if (updatedPrefs) then SlashLib.echoChatAppl(SlashLib.getLocal("Old commands updated."), false, "green") end
	--SlashLib.showHideEditUI(true)
end

---------------------------------------------------------------------------------------
-------------------------------------- SLASH CMD --------------------------------------
function SlashLib.slashCmd(arg, execLevel)
	SlashLib.slashCmd_Lib(arg, execLevel)
end

function SlashLib.slashCmd_getList(arg2) -- liste des commandes-slash (voir .lib)
	-- .slashCmd_showList(dontPrint) -- print la liste en print, ou la renvoit en chaine avec dontPrint à true (pour frame cfg)
	local tbl = {
--[[
		["macro"] = {
			{
				["cmd"] = {"f_cleanline"},
				["func"] = function() SlashLib.cleanMarker(SlashLib.entierParDefaut(arg2, 1)) end,
				["label"] = "INTERNAL: pour macro text btn secure",
			},
		},
--]]
		["user"] = {
			{
				["cmd"] = {"show", "hide"},
				["type"] = "",
				["func"] = function() SlashLib.showHide() end, -- switch
				["label"] = SlashLib.getLocal("show/hide commands menu"),
			},
			{
				["cmd"] = {"alpha"},
				["type"] = "number",
				["func"] = function() SlashLib.setAlpha(arg2) end,
				["label"] = SlashLib.getLocal("set frame alpha"),
			},
			{
				["cmd"] = {"edit"},
				["type"] = "",
				["func"] = function() SlashLib.showHideEditUI() end, -- switch
				["label"] = SlashLib.getLocal("open editing interface"),
			},
			{
				["cmd"] = {"import", "macros"},
				["type"] = "GENERAL | PLAYER",
				["func"] = function() SlashLib.importMacros(arg2) end,
				["label"] = SlashLib.getLocal("import macros to SlashLib"),
			},
			{
				["cmd"] = {"restore"},
				["type"] = "GENERAL | PLAYER",
				["func"] = function() SlashLib.restoreSavedVars(arg2) end,
				["label"] = SlashLib.getLocal("restore loaded commands"),
			},
			{
				["cmd"] = {"resetui", "reset"},
				["type"] = "",
				["func"] = function() SlashLib.resetui() end,
				["label"] = SlashLib.getLocal("reset addon interface"),
			},
			{
				["cmd"] = {"resetall"},
				["type"] = "",
				["func"] = function() SlashLib.resetall() end,
				["label"] = SlashLib.getLocal("reset *ALL* addon settings to default"),
			},
			{
				["cmd"] = {"help", "aide", "?"},
				["type"] = "",
				["func"] = function() SlashLib.help() end,
				["label"] = SlashLib.getLocal("display available slash commands"),
			},
		},
		["dev"] = {
			{
				["cmd"] = {"settings"},
				["func"] = function() SlashLib.printAddonSettings() end,
				["label"] = "print saved settings",
			},
			{
				["cmd"] = {"memory", "mem", "lines"},
				["func"] = function() SlashLib.printAddonMemory() end,
				["label"] = "print saved lines",
			},
			{
				["cmd"] = {"data"},
				["func"] = function() SlashLib.printEditZone() end,
				["label"] = "print edit zone temporary data",
			},
			{
				["cmd"] = {"test"},
				["func"] = function() SlashLib.fcnTest(arg2) end,
				["label"] = "DEV test with 1 argument",
			},
			{
				["cmd"] = {"debug"},
				["type"] = "boolean",
				["func"] = function() SlashLib.toggleDebug() end,
				["label"] = "toggle debug",
			},
		},
		["DEFAULTCMD"] = "help", -- commande par défaut
		["DEFAULTARG"] = "help", -- argument par défaut d'une commande
	}
	return tbl
end

---------------------------------------------------------------------------------------
--------------------------------------- ACTIONS ---------------------------------------
function SlashLib.help()
--[[
	local cfgFrame = Ck_cfg_frame -- /////
	local success, errorMessage = pcall(Settings.OpenToCategory, addonName)
	if (not success) then
		InterfaceOptionsFrame_OpenToCategory(cfgFrame) -- wow9
		InterfaceOptionsFrame_OpenToCategory(cfgFrame) -- wow9 (doublé pour ouvrir la page de l'addon)
	end
--]]
	SlashLib.slashCmd_showList(false) -- pas d'UI (mettre à true pour ne pas printer la liste)
end

function SlashLib.fcnTest(arg) -- $$$--%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	print(date("%y.%m.%d %H:%M:%S", GetServerTime()).."===================================")
end
	
function SlashLib.printAddonSettings() -- dvlpt
	SlashLib.echoChatAppl("per char settings", true, "grey")
	for key, value in pairs(SLIB_SAVED_PERCHAR["settings"]) do
		if (type(value) == "table") then
			SlashLib.echo("    "..key..":", "SYSTEM")
			for subkey, subvalue in pairs(value) do
				if (type(subvalue) == "table") then value = "TABLE" end
				SlashLib.echo("        "..subkey..": "..subvalue, "SYSTEM")
			end
		else
			SlashLib.echo("    "..tostring(key)..": "..tostring(value), "SYSTEM")
		end
	end
end

function SlashLib.printAddonMemory(brutBoolean) -- dvlpt
	if (SlashLib.isTrue(brutBoolean)) then
		SlashLib.echoTable(SLIB_SAVED_DATA["lines"], "lines")
		SlashLib.echoTable(SLIB_SAVED_PERCHAR["lines"], "lines per char")
	else
		SlashLib.echoChatAppl("memory general", true, "grey")
		for key, value in pairs(SLIB_SAVED_DATA["lines"]) do
			SlashLib.echo("       ["..key.."]  index="..value["index"].."    id="..value["idtime"].."    kind="..value["kind"].."   name="..value["name"].."   cmd="..SlashLib.truncStr(value["cmd"], 25), "SYSTEM")
		end
		SlashLib.echoChatAppl("memory per char", true, "grey")
		for key, value in pairs(SLIB_SAVED_PERCHAR["lines"]) do
			SlashLib.echo("       ["..key.."]  index="..value["index"].."    id="..value["idtime"].."    kind="..value["kind"].."   name="..value["name"].."   cmd="..SlashLib.truncStr(value["cmd"], 25), "SYSTEM")
		end
	end
end

function SlashLib.printEditZone() -- dvlpt
	SlashLib.echoDebug(SlashLibZone_Edit.editTbl, "---------- frameEditZone.editTbl")
end

function SlashLib.toggleDebug()
	SLIB_SAVED_DATA["settings"]["debug"] = 1 - tonumber(SLIB_SAVED_DATA["settings"]["debug"])
	SlashLib.echoChatAppl("debug mode "..SlashLib.booleanToStr(SlashLib.debug())..".", false, "orange")
end

function SlashLib.importMacros(arg)
	local arg = arg or ""

	local function importAddMacro(index, idTime, numCmds, perCharBoolean)
		local perCharBoolean = SlashLib.isTrue(perCharBoolean)
		local name, _, body, _ = GetMacroInfo(index)
		if ((not SlashLib.isEmpty(name)) and (not SlashLib.isEmpty(body))) then
			SlashLib.echoChatAppl(SlashLib.getLocal("Importing macro")..": "..SlashLib.kwote(SlashLib.videSiNul(name)), false, "green")
			return SlashLib.getNewMacro(idTime, index + numCmds, 2, perCharBoolean, name, body, true)
		else
			return nil
		end
	end
	
	local kind = SlashLib.getLocal("General")
	local numglobal, numperchar = GetNumMacros()
	local cpt = 0
	local now = SlashLib.getSeconds(true)
	if (string.upper(arg) == "PLAYER") then
		kind = UnitName("Player")
		local numCmds = #SLIB_SAVED_PERCHAR["lines"]
		if (numperchar > 0) then
			for i = 121, (120 + numperchar) do -- Slots 1 through 120 are general macros; 121 through 138 are per-character macros
				now = now + 0.002
				local tbl = importAddMacro(i, now, numCmds, true)
				if (tbl ~= nil) then
					table.insert(SLIB_SAVED_PERCHAR["lines"], tbl)
					cpt = cpt + 1
				end
			end
		end
		if (cpt > 0) then SlashLib.updateScrollPerChar() end
	else
		local numCmds = #SLIB_SAVED_DATA["lines"]
		for i = 1, numglobal do -- Slots 1 through 120 are general macros; 121 through 138 are per-character macros
			now = now + 0.01
			local tbl = importAddMacro(i, now, numCmds, false)
			if (tbl ~= nil) then
				table.insert(SLIB_SAVED_DATA["lines"], tbl)
				cpt = cpt + 1
			end
		end
		if (cpt > 0) then SlashLib.updateScrollGeneral() end
	end
	if (cpt > 0) then
		SlashLib.echoChatAppl(cpt.." "..kind.." "..SlashLib.getLocal("macro(s) imported."), false, "green")
		SlashLib.showHideEditUI(true)
	else
		SlashLib.echoChatAppl(SlashLib.getLocal("No macro found."), false, "orange")
	end
end

function SlashLib.restoreSavedVars(arg)
	local arg = arg or ""
	local kind = SlashLib.getLocal("General")
	if (string.upper(arg) == "PLAYER") then
		SLIB_SAVED_PERCHAR["lines"] = slibSavedPerchar
		SlashLib.updateScrollPerChar()
		kind = UnitName("Player")
	else
		SLIB_SAVED_DATA["lines"] = slibSaved
		SlashLib.updateScrollGeneral()
	end
	SlashLib.echoChatAppl(kind.." "..SlashLib.getLocal("commands restored."), false, "green")
	SlashLib.showHideEditUI(true)
end

function SlashLib.resetui()
	SLIB_SAVED_PERCHAR["settings"] = SlashLib.initDataPerChar(true)
	SlashLibFrameMain:ClearAllPoints()
	SlashLibFrameMain:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", SLIB_SAVED_PERCHAR["settings"]["posX"], SLIB_SAVED_PERCHAR["settings"]["posY"])
	SlashLib.showHide(true)
end

function SlashLib.resetall()
	SlashLib.confirmWarning("SlashLib.resetallConfirmed()")
end

function SlashLib.resetallConfirmed()
	wipe(SLIB_SAVED_DATA)
	SLIB_SAVED_DATA = SlashLib.initData()
	wipe(SLIB_SAVED_PERCHAR)
	SLIB_SAVED_PERCHAR = SlashLib.initDataPerChar()

	SlashLib.resetui()
	SlashLib.updateScrollGeneral()
	SlashLib.updateScrollPerChar()
	--if (_G[addonVar["minimap"]] ~= nil) then SlashLib.minimapOnOff(SLIB_SAVED_DATA["settings"]["minimap"]["visible"]) end
end

function SlashLib.minimapOnOff(optionalBoolean) -- UNUSED
	local val
	local frame = _G[addonVar["minimap"]]
	if (optionalBoolean == nil) then
		----------- switch
		val = (not frame:IsVisible())
	else
		----------- given value
 		val = SlashLib.isTrue(optionalBoolean, true)
	end
	
	if (val) then frame:Show() else frame:Hide() end
	SLIB_SAVED_DATA["settings"]["minimap"]["visible"] = SlashLib.booleanToInt(val)
end

function SlashLib.minimapIsVisible() -- renvoit true si un bouton minimap existe ET visible
	if (_G[addonVar["minimap"]] ~= nil) then
		return _G[addonVar["minimap"]]:IsVisible()
	else
		return false
	end
end

function SlashLib.showHide(optionalBoolean, commHiddenBoolean)
	local val
	if (optionalBoolean == nil) then
		----------- switch
		val = (not SlashLibFrameMain:IsVisible())
	else
		----------- given value
 		val = SlashLib.isTrue(optionalBoolean, true)
	end
	
	if (val) then SlashLibFrameMain:Show() else SlashLibFrameMain:Hide() end
	SLIB_SAVED_PERCHAR["settings"]["visible"] = SlashLib.booleanToInt(val)
	
	if ((not SlashLibFrameMain:IsVisible()) and SlashLib.isTrue(commHiddenBoolean)) then
		SlashLib.echoChatAppl(SlashLib.getLocal("hidden. Type /slashlib show to show commands menu again."), false, "yellow")
	end
end

function SlashLib.setAlpha(number)
	local number = SlashLib.nombreParDefaut(number, 0.5, 0.1, 1)
	SlashLibFrameMain.texture:SetAlpha(number)
	SLIB_SAVED_PERCHAR["settings"]["alpha"] = number
end

---------------------------------------------------------------------------------------
------------------------------------ FCNS INTERNES ------------------------------------
function SlashLib.getNewMacro(idtime, index, kind, perchar, name, cmd, visible)
	local tbl = {
		["idtime"] = idtime or SlashLib.getSeconds(true),
		["index"] = index or 1,
		["kind"] = kind or 2,
		["perchar"] = SlashLib.isTrue(perchar), -- General par défaut
		["name"] = name or "",
		["cmd"] = cmd or "",
		["visible"] = true,--SlashLib.isTrue(visible),
	}
	return tbl
end

function SlashLib.saveSettings()
--[[ pas besoin : save fait en direct
	SLIB_SAVED_PERCHAR["settings"]["visible"] = SlashLib.booleanToInt(SlashLibFrameMain:IsVisible())
	SlashLib.saveFramePosition()
--]]
end

function SlashLib.saveFramePosition()
	SLIB_SAVED_PERCHAR["settings"]["posX"] = floor(SlashLibFrameMain:GetLeft() + 0.5)
	SLIB_SAVED_PERCHAR["settings"]["posY"] = floor(SlashLibFrameMain:GetTop() + 0.5)
	SlashLib.testPrint("SlashLib.saveFramePosition: posX="..SLIB_SAVED_PERCHAR["settings"]["posX"].." posY="..SLIB_SAVED_PERCHAR["settings"]["posY"])
end

function SlashLib.saveMinimapPosition() -- UNUSED
	if (_G[addonVar["minimap"]] ~= nil) then
		SLIB_SAVED_DATA["settings"]["minimap"]["posX"] = floor(_G[addonVar["minimap"]]:GetLeft() + 0.5)
		SLIB_SAVED_DATA["settings"]["minimap"]["posY"] = floor(_G[addonVar["minimap"]]:GetTop() + 0.5)
	end
end

function SlashLib.sortCmds(tblKind) -- modifie les listes (ajouter tri par index)
	local tblKind = tblKind or ""
	if (string.upper(tblKind) == "PERCHAR") then
		SLIB_SAVED_PERCHAR["lines"] = SlashLib.sortTable(SLIB_SAVED_PERCHAR["lines"], "name", "A", "A")
	else
		SLIB_SAVED_DATA["lines"] = SlashLib.sortTable(SLIB_SAVED_DATA["lines"], "name", "A", "A")
	end
end

function SlashLib.execCmd(cmdStr, cmdKindIndex) -- cmdKindIndex : 1 ou 2
	local cmdStr = cmdStr or ""
	SlashLib.testPrint("executeCmd(): cmdStr="..cmdStr.."---cmdKindIndex="..cmdKindIndex.."---")
	if (cmdStr ~= "") then
		SlashLib.executeCmd(cmdStr, cmdKind[cmdKindIndex]["label"] == "MACRO") -- lib
	else
		SlashLib.echoChatAppl("command is empty.", false, "red")
	end
end

function SlashLib.getCmdByName(cmdName, optionalTbl) -- renvoit la sous-table de la commande passée
	-- si optionalTbl est vide : recherche dans les tables général et perChar
	-- peut retourner une table vide !!!

	local function getCmdByNameFind(cmdName, tbl) -- recherche de cmdName dans tbl passée : peut retourner une table vide !!!
		local tblResult = {}
		for i, value in ipairs(tbl) do
			if (strtrim(value.name) == cmdName) then
				tblResult = value
				break
			end
		end
		return i, tblResult
	end
	
	local index = 0
	local tblCmd = {}
	if (not SlashLib.isEmpty(cmdName)) then
		cmdName = strtrim(cmdName)
		if (type(optionalTbl) ~= "table") then
			-------------------------------- peuplement table temporaire
			local tbl = {}
			for i, value in ipairs(SLIB_SAVED_DATA["lines"]) do
				table.insert(tbl, value)
			end
			for i, value in ipairs(SLIB_SAVED_PERCHAR["lines"]) do
				table.insert(tbl, value)
			end
			index, tblCmd = getCmdByNameFind(cmdName, tbl)
		else
			index, tblCmd = getCmdByNameFind(cmdName, optionalTbl)
		end
	end

	return index, tblCmd -- tblCmd peut être une table vide !!!
end

---------------------------------------------------------------------------------------
------------------------------------- LIB DEDIEE --------------------------------------
function SlashLib.debug() -- raccourci : renvoit true si debug ON
	return SlashLib.isTrue(SLIB_SAVED_DATA["settings"]["debug"])
end

function SlashLib.getVar(nomVar, subLvl1, subLvl2) -- nécessite addonVar
	-- get valeurs de la table locale addonVar
	-- si aucun argument n'est passé, renvoit la table addonVar
	if (addonVar == nil) then return nil end
	
	if (subLvl2 ~= nil) then
		if (addonVar[nomVar][subLvl1][subLvl2] ~= nil) then return addonVar[nomVar][subLvl1][subLvl2] end
	elseif (subLvl1 ~= nil) then
		if (addonVar[nomVar][subLvl1] ~= nil) then return addonVar[nomVar][subLvl1] end
	elseif (nomVar ~= nil) then
		if (addonVar[nomVar] ~= nil) then return addonVar[nomVar] end
	else
		return addonVar
	end
end

function SlashLib.setVar(nomVar, valeur) -- nécessite addonVar
	if (addonVar == nil) then return nil end
	if (nomVar ~= nil) then
		if (addonVar[nomVar] ~= nil) then addonVar[nomVar] = valeur end
	end
end

function SlashLib.getSavedSettings(perCharacterBoolean) -- pour appel depuis autres fichiers
	if (perCharacterBoolean) then return SLIB_SAVED_PERCHAR else return SLIB_SAVED_DATA end
end

---------------------------------------------------------------------------------------
------------------------------------------ UI -----------------------------------------
function SlashLib.ui_createMainFrame() -- modif main frame : modif main framehidden par XML
	SlashLibFrameMain:SetFrameStrata("MEDIUM") -- pour passer au dessus des autres frames courants
	SlashLibFrameMain:SetClampedToScreen(true)
	SlashLibFrameMain:SetMovable(true)
 	SlashLibFrameMain:SetUserPlaced(true)
    SlashLibFrameMain:SetResizable(false)
	SlashLibFrameMain:ClearAllPoints()
	SlashLibFrameMain:SetSize(addonVar["btnWidth"], 20)
	SlashLibFrameMain:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", SlashLib.entierParDefaut(SLIB_SAVED_PERCHAR["settings"]["posX"], SlashLib.initDataPerChar(true, "posX")), SlashLib.entierParDefaut(SLIB_SAVED_PERCHAR["settings"]["posY"], SlashLib.initDataPerChar(true, "posY")))
	
	SlashLib.setTexturedFrame(SlashLibFrameMain, "SlashLibFrameMain_tex", SlashLib.getStuffPath("btn", true), 90, 25, 1)
	SlashLibFrameMain.texture:SetAlpha(SLIB_SAVED_PERCHAR["settings"]["alpha"])
	
	---- créé une TEXTURE par FICHIER IMAGE au frame passé (et renvoit un pointeur vers cette texture)
	---- texName : nom de texture (si nil, ajout de "Hi")
	---- imgCoordH et imgCoordV : tables  {h1, v1} et {v1, v2}, ou entiers si pas besoin de décalage
	---- hilitedAlphaOrRGB (seulement pour les boutons) : texture hilited (autre image ou valeur alpha)
	---- attention : le SetSize() du frame n'est pas inclus !
	--[[ exemples d'appel :
			SlashLib.setTexturedFrame(myFrame, "myFrame_tex", "Interface\\AddOns\\NPCsMarkers\\stuff\\myPict", 40, 25, 0.8, "Interface\\AddOns\\NPCsMarkers\\stuff\\myPictHI") -- size frame : 40 x 25
			SlashLib.setTexturedFrame(myFrame, nil, "Interface\\AddOns\\NPCsMarkers\\stuff\\myPict", {10, 40}, {0, 30}, 1, 0.3)
	--]]
	
	--------------------------------------------------------- init POPUP MENU
	local framePopupMenu = SlashLib.ui_initializePopupMenu()
	
	--------------------------------------------------------- bouton principale
	local btnMain = CreateFrame("Button", "SlashLib_mainBtn", SlashLibFrameMain)
	btnMain:SetSize(SlashLibFrameMain:GetWidth() - 6, SlashLibFrameMain:GetHeight() - 6)
	btnMain:SetPoint("CENTER", SlashLibFrameMain, "CENTER", 0, 0)
	--------------------------------- texte
	btnMain.text = btnMain:CreateFontString("$parent_Text", "OVERLAY", "GameFontNormalSmall")
	SlashLib.setFontStyle(btnMain.text, SlashLib.getStuffPath("Myriad Condensed Web.ttf", true), 11, "yellow", false, 1)
	btnMain.text:SetPoint("LEFT", "$parent", "LEFT", 4, 0)
	btnMain.text:SetJustifyH("CENTER")
	btnMain.text:SetText(addonName)
	--------------------------------- tooltip
    btnMain:SetScript("OnEnter", function(self)
		local hiAlpha = 1
		if (SLIB_SAVED_PERCHAR["settings"]["alpha"] > 0.85) then hiAlpha = 0.6 end
		SlashLibFrameMain.texture:SetAlpha(hiAlpha)
		GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT", -50, 4)
		GameTooltip:AddLine("Right-click to edit commands library.".."\n".."Drag with shift or alt to move this frame.")
		GameTooltip:Show()
    end)
    btnMain:SetScript("OnLeave", function(self)
		SlashLibFrameMain.texture:SetAlpha(SLIB_SAVED_PERCHAR["settings"]["alpha"])
		GameTooltip:Hide()
	end)
	--------------------------------- drag déplacement et clicks
	btnMain:SetScript("OnMouseDown", function(self, mouse)
		if ((mouse == "LeftButton" and IsModifierKeyDown()) and not SlashLibFrameMain.isMoving) then
			SlashLibFrameMain:StartMoving()
			SlashLibFrameMain.isMoving = true
		end
	end)
	btnMain:SetScript("OnMouseUp", function(self, mouse)
		if (SlashLibFrameMain.isMoving) then
			SlashLibFrameMain:StopMovingOrSizing()
			SlashLibFrameMain.isMoving = false
			SlashLib.saveFramePosition()
		else
			if (mouse == "LeftButton") then -- gauche : appel POPUP MENU
				ToggleDropDownMenu(1, nil, framePopupMenu, "SlashLib_mainBtn", -20, 1)
				--SlashLib.flash(SlashLibFrameMain.texture, 1)
			elseif (mouse == "RightButton") then -- doite : Edit
				SlashLib.showHideEditUI()
			end
		end
	end)
	
	---------------------------------------------------------
	if (SLIB_SAVED_PERCHAR["settings"]["visible"] == 1) then SlashLibFrameMain:Show() end
end

function SlashLib.ui_initializePopupMenu() -- création POPUP MENU
	local frame = CreateFrame("Frame", "SlashLib_popupMenu", nil, "UIDropDownMenuTemplate")
	frame.displayMode = "MENU"
	frame.initialize = function(self, level)
		local menuContent = {}
		menuContent.padding = 10 -- /////
		
		menuContent.notCheckable = true
		menuContent.disabled = nil
		menuContent.isTitle = true
		menuContent.text = SlashLib.getLocal("Execute a command:")
		UIDropDownMenu_AddButton(menuContent)
		
		menuContent.isTitle = nil
		menuContent.disabled = nil
		
		menuContent.colorCode = "|cff"..addonVar["colorGeneral"]
		SlashLib.sortCmds()
		for i, value in ipairs(SLIB_SAVED_DATA["lines"]) do
			if (value.visible) then
				menuContent.text = "    "..value.name
				menuContent.func = function(arg1, arg2) -- arg1 renvoit le nom du frame invisible et quelques propriétés
					SlashLib.ui_runMenuCmd(arg1.value, SLIB_SAVED_DATA["lines"])
				end
				UIDropDownMenu_AddButton(menuContent)
			end
		end
		
		menuContent.colorCode = "|cff"..addonVar["colorPerChar"]
		SlashLib.sortCmds("perchar")
		for i, value in ipairs(SLIB_SAVED_PERCHAR["lines"]) do
			if (value.visible) then
				menuContent.text = "    "..value.name
				menuContent.func = function(arg1, arg2) -- arg1 renvoit le nom du frame invisible et quelques propriétés
					SlashLib.ui_runMenuCmd(arg1.value, SLIB_SAVED_PERCHAR["lines"])
				end
				UIDropDownMenu_AddButton(menuContent)
			end
		end
		
		menuContent.colorCode = "|cffC87831"
		menuContent.text = SlashLib.getLocal("Edit commands list...")
		menuContent.func = function()
			SlashLib.showHideEditUI(true)
		end
		UIDropDownMenu_AddButton(menuContent)
		
		menuContent.colorCode = "|cffAAAADD"
		menuContent.text = SlashLib.getLocal("Cancel")
		menuContent.func = function() end
		UIDropDownMenu_AddButton(menuContent)
	end
	return frame
end

function SlashLib.ui_runMenuCmd(cmdStr, optionalTbl) -- si optionalTbl omis, recherche dans les 2 tables (général et perChar)
	SlashLib.testPrint("ui_runMenuCmd() ", cmdStr)
	if (cmdStr ~= nil) then
		local index, tblCmd = SlashLib.getCmdByName(cmdStr, optionalTbl)
		if (tblCmd.cmd ~= nil) then SlashLib.execCmd(tblCmd.cmd, tblCmd.kind) end
	end
end

function SlashLib.confirmWarning(strScriptOnAccepted)
	strScriptOnAccepted = strScriptOnAccepted or nil
	
	local str = SlashLib.getLocal("Reset all settings to default?").."\n\n"..SlashLib.getTxtColor(SlashLib.getLocal("All of your saved slash-commands will be permanently deleted!"), "red")
	SlashLib.showAlert("|cFF"..addonVar["chatColor"]..addonVar["name"].." "..addonVar["version"].."|r\n\n\n"..str, strScriptOnAccepted)
end

function SlashLib.showAlert(msg, strScriptOnAccepted) -- simple alert
	StaticPopupDialogs["SLASHLIB_ALERT"] = {
		text = "%s".."\n ",
		button1 = "Reset", -- ACCEPT,
		button2 = "No", -- CANCEL,
		OnAccept = function(self)
			--print("accepted")
			if (strScriptOnAccepted ~= nil) then RunScript(strScriptOnAccepted) end
		end,
		OnCancel = function(self)
			--print("cancelled")
		end,
		showAlert = 1, -- icon warning
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1,
		preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/
	};
	StaticPopup_Show("SLASHLIB_ALERT", msg) 
end