local frameGeneralList -- scroll frame liste
local framePerCharList -- scroll frame liste
local frameSlashList -- scroll frame toutes slash cmds
local frameEditZone

function SlashLib.blur(box)
	box:HighlightText(0, 0) -- dé-select
	box:ClearFocus() -- de-focus
end

function SlashLib.setEditData(tblData, numLiEdited) -- propriété frame pour édition macro
	numLiEdited = numLiEdited or 0
	if (type(tblData) ~= "table") then tblData = SlashLib.getNewMacro() end
	frameEditZone.editTbl = tblData
	frameEditZone.editTbl["line"] = numLiEdited -- si > 0 : correspond à la ligne de SLIB_SAVED (general ou perchar)
	if (SlashLib.debug()) then SlashLib.printEditZone() end -- /////
end

function SlashLib.showHideEditUI(optionalBoolean)
	local val
	if (optionalBoolean == nil) then
		----------- switch
		val = (not SlashLibFrameEdit:IsVisible())
	else
		----------- given value
 		val = SlashLib.isTrue(optionalBoolean, true)
	end
	
	if (val) then
		SlashLibFrameEdit:Show()
		SlashLib.updateScrollGeneral()
		SlashLib.updateScrollPerChar()
		SlashLib.scrollUpdate(frameSlashList, frameSlashList.allCmds) -- peuplement liste all available commandes
	else
		SlashLibFrameEdit:Hide()
		SlashLib.setEditData() -- reset
	end
end

function SlashLib.updateScrollGeneral()
	SlashLib.sortCmds()
	if (SlashLibFrameEdit:IsVisible()) then SlashLib.scrollUpdate(frameGeneralList, SLIB_SAVED_DATA["lines"]) end
end

function SlashLib.updateScrollPerChar()
	SlashLib.sortCmds("perchar")
	if (SlashLibFrameEdit:IsVisible()) then SlashLib.scrollUpdate(framePerCharList, SLIB_SAVED_PERCHAR["lines"]) end
end

---------------------------------------------------------------------------------------
--------------------------------------- UI EDITION ------------------------------------

function SlashLib.createEditUI()
	local uiWidth = SlashLib.getVar("widthUI")
	local mainFrame = CreateFrame("Frame", "SlashLibFrameEdit", UIParent)
	SlashLibFrameMain:SetFrameStrata("MEDIUM")
	mainFrame:Hide()
	mainFrame:SetClampedToScreen(true)
	mainFrame:SetMovable(true)
 	mainFrame:SetUserPlaced(true)
    mainFrame:SetResizable(false)
	mainFrame:ClearAllPoints()
	mainFrame:SetSize(uiWidth, 16) -- 16 = hauteur barre de titre
	--mainFrame:SetPoint(...) : placement à la fin
	SlashLib.setEscapeClosable(mainFrame)

	-------------------------------------------- barre de titre (SlashLibFrameMenu)
	local menubar = CreateFrame("Button", "SlashLibFrameMenu", mainFrame)
	menubar:SetSize(uiWidth, 16)
	menubar:SetPoint("TOPLEFT", mainFrame, "TOPLEFT", 0, 0)
	SlashLib.setColoredButton(menubar, {0.31,0.38,0.56}, nil, 0, SlashLib.getVar("alphaUI"), 0.3)
	------------- titre
	local strTitle = menubar:CreateFontString("$parent_Text", "OVERLAY", "GameFontNormalSmall")
	SlashLib.setFontStyle(strTitle, "arial", 11, {1, 0.8, 0})
	strTitle:SetPoint("CENTER",0,0)
	strTitle:SetText(SlashLib.getVar("name").." "..SlashLib.getVersion(false).." - "..SlashLib.getLocal("Editing"))
	strTitle:SetJustifyH("CENTER")
	strTitle:SetTextColor(0.92, 0.8, 0.1, 1)
	
	------------- drag déplacement (clic-gauche ou droit)
	menubar:EnableMouse(true)
	menubar:SetScript("OnMouseDown", function(self, mouse)
		if (not self:GetParent().isMoving) then
			self:GetParent():StartMoving()
			self:GetParent().isMoving = true
		end
	end)
	menubar:SetScript("OnMouseUp", function(self, mouse)
		if (self:GetParent().isMoving) then
			self:GetParent():StopMovingOrSizing()
			self:GetParent().isMoving = false
		end
	end)
	
	-------------------------------------------- case de fermeture
	local btnClose = CreateFrame("Button", "SlashLibBtnClose", menubar)
	btnClose:SetPoint("TOPRIGHT", menubar, "TOPRIGHT", 0, 0)
	btnClose:SetSize(30, 16)
	SlashLib.setColoredButton(btnClose, {0.66,0.28,0.19}, nil, 0, SlashLib.getVar("alphaUI"), 0.4)
	-------------- texte
	local strBtn = btnClose:CreateFontString("$parent_Text", "OVERLAY", "GameFontNormalSmall")
	strBtn:SetPoint("CENTER", 1, 0)
	strBtn:SetText("X")
	strBtn:SetJustifyH("CENTER")
	------------- script
	btnClose:SetScript("OnClick", function()
		SlashLib.showHideEditUI(false)
	end)
	
	-------------------------------------------- container principal (sous la menubar) : masquée quand UI repliée
	local frameUI = CreateFrame("Frame", "SlashLibUI", mainFrame) -- zone 
	frameUI:SetWidth(uiWidth) -- height def à la fin
	SlashLib.setColoredFrame(frameUI, {0.2, 0.2, 0.2}, nil, 0, SlashLib.getVar("alphaUI"))
	frameUI:SetPoint("TOPLEFT", menubar, "BOTTOMLEFT", 0, 0)
	
	-------------------------------------------- listes
	local margin = SlashLib.getVar("margin")
	local widthList = SlashLib.getVar("widthList")
	----------------------------- zone "General"
	local frameGeneral = CreateFrame("Frame", "SlashLibZone_General", frameUI)
	frameGeneral:SetPoint("TOPLEFT", frameUI, "TOPLEFT", margin, 0 - margin)
	frameGeneral:SetWidth(widthList - margin) -- hauteur def plus bas
	----------------- label
	local fLabel = SlashLib.createLabelFrame("General", frameGeneral)
	----------------- liste scroll
	frameGeneralList = CreateFrame("Frame", "SlashLibList_General", frameGeneral)
	frameGeneralList:SetPoint("TOPLEFT", fLabel, "BOTTOMLEFT", 0, 0)
	frameGeneralList:SetWidth(widthList) -- hauteur def par nbre lignes visibles
	SlashLib.setColoredFrame(frameGeneralList, "000000", "222222", 1, 0.4)
	local scrollProps, scrollColumns = SlashLib.getListProps()
	SlashLib.scrollCreate(frameGeneralList, scrollProps, scrollColumns) -- creates structure (empty scroll lines)
	
	----------------------------- zone "PerChar"
	local framePerChar = CreateFrame("Frame", "SlashLibZone_PerChar", frameUI)
	framePerChar:SetPoint("TOPLEFT", frameGeneral, "BOTTOMLEFT", 0, 0 - margin)
	framePerChar:SetWidth(widthList - margin) -- hauteur def plus bas
	----------------- label
	fLabel = SlashLib.createLabelFrame("PerChar", framePerChar)
	----------------- liste scroll
	framePerCharList = CreateFrame("Frame", "SlashLibList_PerChar", framePerChar)
	framePerCharList:SetPoint("TOPLEFT", fLabel, "BOTTOMLEFT", 0, 0)
	framePerCharList:SetWidth(widthList) -- hauteur def par nbre lignes visibles
	SlashLib.setColoredFrame(framePerCharList, "000000", "222222", 1, 0.4)
	local scrollProps, scrollColumns = SlashLib.getListProps()
	SlashLib.scrollCreate(framePerCharList, scrollProps, scrollColumns) -- creates structure (empty scroll lines)
	
	--------------------------- ajustements hauteurs zones
	frameGeneral:SetHeight(fLabel:GetHeight() + frameGeneralList:GetHeight())
	framePerChar:SetHeight(frameGeneral:GetHeight())
	
	-------------------------------------------- zone édition
	SlashLib.createEditZone(frameUI, framePerChar)
	
	-------------------------------------------- réglage finale hauteur frameUI
	frameUI:SetHeight(margin + frameGeneral:GetHeight() + margin + framePerChar:GetHeight() + margin + SlashLibZone_Edit:GetHeight() + margin)
	mainFrame:SetPoint("TOP", UIParent, "TOP", 0, -100) -- position fixe (pas de prefs)
	
	-------------------------------------------- liste slashs built-in
	SlashLib.createMacrosFrame(frameUI, uiWidth - widthList, frameUI:GetHeight() - (margin * 2))
	frameSlashList:SetPoint("TOPRIGHT", frameUI, "TOPRIGHT", 0 - margin, 0 - margin)
end

---------------------------------------------------------------------------------------
------------------------------------ ZONE EDITION -------------------------------------

function SlashLib.createEditZone(parent, relFrame)
	local editZoneHeight = 150 -- /////
	frameEditZone = CreateFrame("Frame", "SlashLibZone_Edit", parent)
	frameEditZone:SetPoint("TOPLEFT", relFrame, "BOTTOMLEFT", 0, 0 - SlashLib.getVar("margin"))
	frameEditZone:SetSize(relFrame:GetWidth(), editZoneHeight)
	SlashLib.setColoredFrame(frameEditZone, "112222", "555555", 1, 0.5)
	
	local boxEditName = SlashLib.drawEditBox(SlashLib.getLocal("Name"), "name", frameEditZone, 200, 20, 10)
	boxEditName:SetScript("OnTabPressed", function(self) SlashLibZone_Edit_EditBox_cmd:SetFocus() end)
	
	local boxEditCmd = SlashLib.drawEditBox(SlashLib.getLocal("Command"), "cmd", frameEditZone, 440, 60, 35)
	boxEditCmd:SetScript("OnTabPressed", function(self) SlashLibZone_Edit_EditBox_name:SetFocus() end)

	------------------------------------------- boutons check
	local fontTbl = {["font"] = "arial", ["size"] = 12, ["color"] = SlashLib.getVar("colorPerChar")}
	local btnPerChar = SlashLib.drawCheckButton("SlashLibZone_Edit_Perchar", UnitName("Player"), "848442", fontTbl, frameEditZone, false)
	btnPerChar:SetPoint("LEFT", boxEditName, "RIGHT", 50, 0)
	
	------------------------------------------- boutons
	local btnMargin = 0
	local btnNew = SlashLib.drawButton("$parent_btnNew", frameEditZone, SlashLib.getLocal("New"), 90, boxEditCmd, "LEFT", btnMargin, {0.49,0.4,0.07}, nil)
	btnNew:SetScript("OnClick", function() SlashLib.btnMacroNew(true) end)
	
	local btnSave = SlashLib.drawButton("$parent_btnSave", frameEditZone, SlashLib.getLocal("Save"), 90, boxEditCmd, "RIGHT", btnMargin, {0.49,0.4,0.07}, nil)
	btnSave:SetScript("OnClick", function() SlashLib.btnMacroSave() end)
	btnMargin = btnMargin + btnSave:GetWidth() + 30
	
	local btnTest = SlashLib.drawButton("$parent_btnTest", frameEditZone, SlashLib.getLocal("Test"), 70, boxEditCmd, "RIGHT", btnMargin, {0.49,0.4,0.07}, nil)
	btnTest:SetScript("OnClick", function() SlashLib.btnMacroTest() end)
	btnMargin = btnMargin + btnTest:GetWidth() + 30
	
	local btnDelete = SlashLib.drawButton("$parent_btnDelete", frameEditZone, SlashLib.getLocal("Delete"), 90, boxEditCmd, "RIGHT", btnMargin, {0.59,0.17,0.06}, nil)
	btnDelete:SetScript("OnClick", function() SlashLib.btnMacroDelete() end)
	btnMargin = btnMargin + btnDelete:GetWidth() + 30
	
	SlashLib.updateSaveButton()
	SlashLib.setEditData() -- init propriété pour Edition macro
	
	frameEditZone.eyes = {
		["visible"] = SlashLib.getStuffPath("eye_1.tga", true),
		["invisible"] = SlashLib.getStuffPath("eye_0.tga", true),
	}
end

function SlashLib.drawCheckButton(btnName, label, bkColor, fontTbl, parent, checked) 
	-- btn.checked disponible (propriété)
	-- btn.setChecked(boolean) disponible
	-- btn.setLabel(labelStr) disponible
	-- btn.setLabelColor(color) disponible
	local marginH = 6
	-------------------------------------------- btn (ensemble : clic sur label = clic check box)
	local btn = CreateFrame("Button", btnName, parent)
	btn:SetSize(16, 16) -- modif width plus bas
	------------------------- case à cocher
	local checkBox = CreateFrame("Frame", btnName.."_chkBox", btn) -- un Frame pour passer le clic !
	checkBox:SetSize(16, 16)
	SlashLib.setColoredFrame(checkBox, bkColor, "111111", 1, SlashLib.getVar("alphaUI"), 0.2)
	checkBox:SetPoint("LEFT", btn, "LEFT", 0, 0)
	local strCheck = checkBox:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	strCheck:SetFont(SlashLib.getStuffPath("arial", true), 12, "OUTLINE")
	strCheck:SetTextColor(0.86, 0.84, 0.49)
	strCheck:SetPoint("CENTER", 1, 0)
	strCheck:SetJustifyH("CENTER")
	------------------------- label
	local fLabel = CreateFrame("Button", btn:GetName().."_Label", btn)
	fLabel:SetSize(1, btn:GetHeight()) -- largeur très fine pour ne pas gêner le clic
	fLabel:SetPoint("LEFT", checkBox, "RIGHT", marginH, 0)
	------------- text
	btn.label = fLabel:CreateFontString(fLabel:GetName().."_Txt", "OVERLAY", "GameFontNormal")
	SlashLib.setFontStyle(btn.label, fontTbl.font, fontTbl.size, fontTbl.color)
	btn.label:SetPoint("LEFT", 0, 0)
	btn.label:SetJustifyH("LEFT")
	btn.label:SetText(label)
	-------------------------------------------- fcns
	btn.checked = SlashLib.isTrue(checked)
	btn.setChecked = function(boolean)
		btn.checked = SlashLib.isTrue(boolean)
		strCheck:SetText("")
		if (btn.checked) then strCheck:SetText("X") end
	end
	btn.setLabel = function(labelStr) btn.label:SetText(labelStr) end
	btn.setLabelColor = function(color) SlashLib.setFontStyle(btn.label, fontTbl.font, fontTbl.size, color) end
	-------------------------------------------- scripts
	btn:SetScript("OnClick", function(self)
		self.setChecked(not self.checked)
		SlashLib.updateSaveButton(bkColor)-- spécial SlashLib
	end)
	btn:SetWidth(checkBox:GetWidth() + marginH + btn.label:GetStringWidth() + 4)
	-------------------------------------------- 
	return btn
end

function SlashLib.updateSaveButton() -- changement couleur suivant perchar
	local bkColor = SlashLib.getVar("colorGeneral")
	if (SlashLibZone_Edit_Perchar.checked) then bkColor = SlashLib.getVar("colorPerChar") end
	if (SlashLibZone_Edit_btnSave ~= nil) then SlashLib.setColoredButton(SlashLibZone_Edit_btnSave, bkColor, "111111", 1, SlashLib.getVar("alphaUI") - 0.2, 0.4) end
end

function SlashLib.drawButton(btnName, parent, label, width, objAnchor, align, posH, bkColor, alpha) -- dessin d'un bouton standard
	local alpha = alpha or SlashLib.getVar("alphaUI")
	local btn = CreateFrame("Button", btnName, parent)
	if (align == "RIGHT") then posH = 0 - posH end
	btn:SetSize(width, 18)
	btn:SetPoint("TOP"..align, objAnchor, "BOTTOM"..align, posH, -4)
	SlashLib.setColoredButton(btn, bkColor, "111111", 1, alpha, 0.6)
	-------------- texte
	local strBtn = btn:CreateFontString("$parent_Text", "OVERLAY", "GameFontNormalSmall")
	strBtn:SetFont(SlashLib.getStuffPath("Myriad Condensed Web.ttf", true), 12, "NONE")
	strBtn:SetTextColor(1, 0.8, 0) -- jaune standard
	strBtn:SetPoint("CENTER", 1, 0)
	strBtn:SetText(label)
	strBtn:SetJustifyH("CENTER")
	
	return btn
end

function SlashLib.drawEditBox(labelStr, kind, parent, editWidth, editHeight, marginV) -- creation d'une EditBox avec son label
	------------------------- label
	local fLabel = CreateFrame("Frame", "$parent_Label_"..kind, parent)
	fLabel:SetSize(70, 16)
	fLabel:SetPoint("TOPLEFT", parent, "TOPLEFT", 0, 0 - marginV)
	------------- texte
	local str = fLabel:CreateFontString("$parent_Edit_"..kind.."_Txt", "OVERLAY", "GameFontNormal")
	SlashLib.setFontStyle(str, "arial", 11, {0.8, 0.6, 0})
	str:SetPoint("RIGHT", 0, 0)
	str:SetJustifyH("RIGHT")
	str:SetText(labelStr)
	------------------------- editBox
	local editBox = CreateFrame("EditBox", "$parent_EditBox_"..kind, parent)
	editBox:SetSize(editWidth, editHeight) -- height ne fonctionne pas si multiLine !
	local multiLine = (editHeight > 22)
	editBox:SetMultiLine(multiLine)
	if (multiLine) then -- procédure spéciale pour hauteur
		local maxNumLines = 6 -- /////
		editBox:SetPoint("TOPLEFT", fLabel, "TOPRIGHT", 6, 2)
		editBox:SetPoint("BOTTOMRIGHT", parent, "BOTTOMRIGHT", -15, 29)
		editBox:SetMaxLetters(255)
		editBox:SetTextInsets(6, 6, 4, 4) -- left, right, top, bottom
		editBox:SetScript("OnTextChanged", function(self)
			if (#{strsplit("\n", self:GetText())} > maxNumLines) then SlashLib.blur(self) end
		end)
	else
		editBox:SetPoint("LEFT", fLabel, "RIGHT", 6, 0)
		editBox:SetMaxLetters(30)
		editBox:SetTextInsets(6, 6, 0, 0) -- left, right, top, bottom
		editBox:SetScript("OnEnterPressed", function(self) SlashLib.blur(self) end)
	end
	SlashLib.setColoredFrame(editBox, "848442", "222222", 1, 0.5)
	---------------
	editBox:SetAutoFocus(false)
	editBox:SetHighlightColor(0.65,0.69,0.1) -- couleur sélection texte
	editBox:SetFontObject("ChatFontNormal")
	editBox:SetFont(SlashLib.getStuffPath("consola.ttf", true), 12, "")
	editBox:SetText("")
	
	editBox:SetScript("OnEscapePressed", function(self) SlashLib.blur(self) end)
	------------------------- 
	return editBox
end

function SlashLib.createLabelFrame(kind, parent) -- frame de label/titre : kind = "General" ou "PerChar"
	local frame = CreateFrame("Frame", "SlashLibLabel_"..kind, parent)
	frame:SetSize(parent:GetWidth(), 16)
	frame:SetPoint("TOP", parent, "TOP", 0, 0)
	------------- texte
	local str = frame:CreateFontString("$parent_Text", "OVERLAY", "GameFontNormal")
	local strColor = SlashLib.getColor(SlashLib.getVar("color"..kind), "hexa")
	SlashLib.setFontStyle(str, "arial", 13, strColor)
	str:SetPoint("CENTER", 0, 2)
	str:SetJustifyH("CENTER")
	local txt = SlashLib.getLocal("General")
	if (kind ~= "General") then txt = UnitName("player") end
	str:SetText(txt)
	------------- 
	return frame
end

function SlashLib.btnMacroNew(setFocusBoolean)
	SlashLib.fillEditMacro(nil, 0)
	if (SlashLib.isTrue(setFocusBoolean)) then SlashLibZone_Edit_EditBox_name:SetFocus() end -- select
end

function SlashLib.btnMacroSave()
	----------------------------------------------------------- préparation data
	local cmdKindIndex = 2 --/////
	local numLiEdited = frameEditZone.editTbl["line"] -- raccourci : si > 0 : correspond à la ligne de SLIB_SAVED (general ou perchar)
	local newMacro = (numLiEdited == 0)
	local tbl = {}
	tbl["name"] = SlashLibZone_Edit_EditBox_name:GetText()
	if (strtrim(tbl["name"]) ~= "") then
		tbl["cmd"] = strtrim(SlashLibZone_Edit_EditBox_cmd:GetText())
		if (string.len(tbl["cmd"]) >= 1) then
			tbl["perchar"] = SlashLibZone_Edit_Perchar.checked
			tbl["visible"] = frameEditZone.editTbl["visible"]
			tbl["kind"] = cmdKindIndex
			tbl["idtime"] = frameEditZone.editTbl["idtime"] or SlashLib.getSeconds(true)
			if (newMacro) then
				if (tbl["perchar"]) then
					tbl["index"] = #SLIB_SAVED_PERCHAR["lines"] + 1
				else
					tbl["index"] = #SLIB_SAVED_DATA["lines"] + 1
				end
			else
				tbl["index"] = frameEditZone.editTbl["index"]
			end
			--------------------------------------------------- enregistrement
			if (SlashLib.debug()) then -- /////
				SlashLib.echoDebug(tbl, "---------.btnMacroSave()")
				print("-------------------- numLiEdited=", numLiEdited)
			end
			if (tbl["perchar"]) then
				if (newMacro) then
					table.insert(SLIB_SAVED_PERCHAR["lines"], tbl)
				else
					SLIB_SAVED_PERCHAR["lines"][numLiEdited] = tbl
				end
				SlashLib.updateScrollPerChar()
			else
				if (newMacro) then
					table.insert(SLIB_SAVED_DATA["lines"], tbl)
				else
					SLIB_SAVED_DATA["lines"][numLiEdited] = tbl
				end
				SlashLib.updateScrollGeneral()
			end
			SlashLib.echoChatAppl(SlashLib.kwote(tbl["name"]).." "..SlashLib.getLocal("saved."), false, "aqua")
			SlashLib.btnMacroNew(false) -- reset
		else
			SlashLib.echoChatAppl("command is empty.", false, "red")
		end
	else
		SlashLib.echoChatAppl("name is empty.", false, "red")
	end
end

function SlashLib.btnMacroDelete()
	local numLiEdited = frameEditZone.editTbl["line"] -- raccourci : si > 0 : correspond à la ligne de SLIB_SAVED (general ou perchar)
	if (numLiEdited > 0) then
		if (frameEditZone.editTbl["perchar"]) then
			table.remove(SLIB_SAVED_PERCHAR["lines"], numLiEdited)
			SlashLib.updateScrollPerChar()
		else
			table.remove(SLIB_SAVED_DATA["lines"], numLiEdited)
			SlashLib.updateScrollGeneral()
		end
		SlashLib.btnMacroNew(false) -- reset
	else
		SlashLib.echoChatAppl("no saved macro to delete.", false, "red")
	end
end

function SlashLib.btnMacroTest()
	local cmdKindIndex = 2 --/////
	SlashLib.execCmd(SlashLibZone_Edit_EditBox_cmd:GetText(), cmdKindIndex)
end

---------------------------------------------------------------------------------------
-------------------------- LISTES MACROS (GENERAL & PERCHAR)---------------------------

function SlashLib.scrollLineUpdate(scrollFrame, lineIndex, dataLineIndex) -- do not modify args !
	local columnFrame, dataText, bkColor, r, g, b, alpha
	if (scrollFrame == frameSlashList) then
		--------------------------------------------------- liste all available macros
		local thisData = frameSlashList.allCmds[dataLineIndex]
		columnFrame = scrollFrame.line[lineIndex].column[1]
		columnFrame.text:SetText(thisData)
		
	else
		--------------------------------------------------- listes macros user
		local thisData = {}
		local alpha = SlashLib.scrollGetColumnProperties(scrollFrame, 4)["alpha"] or 1
		local r, g, b = SlashLib.getColor(SlashLib.getVar("colorGeneral"), "rgb")
		if (scrollFrame == framePerCharList) then
			r, g, b = SlashLib.getColor(SlashLib.getVar("colorPerChar"), "rgb")
			thisData = SLIB_SAVED_PERCHAR["lines"][dataLineIndex]
		else
			thisData = SLIB_SAVED_DATA["lines"][dataLineIndex]
		end
		------------------------ column 1: text and texture available
		columnFrame = scrollFrame.line[lineIndex].column[1]
		columnFrame.text:SetText(" ") -- ///// btns up and down à faire
		
		------------------------ column 2: text and texture available
		columnFrame = scrollFrame.line[lineIndex].column[2]
		columnFrame.text:SetText(" ") -- ///// btns up and down à faire
		
		------------------------ column 3: text and texture available
		columnFrame = scrollFrame.line[lineIndex].column[3]
		------------ background color & text update
		local icnPath = frameEditZone.eyes.invisible
		if (thisData["visible"]) then icnPath = frameEditZone.eyes.visible end
		columnFrame.texture:SetTexture(icnPath)
		
		------------------------ column 4: text and texture available
		columnFrame = scrollFrame.line[lineIndex].column[4]
		------------ text update
		dataText = thisData["name"]
		columnFrame.text:SetText(dataText)
		------------ background color update
		columnFrame.texture:SetColorTexture(r, g, b, alpha)
		
		------------------------ column 5: text and texture available
		columnFrame = scrollFrame.line[lineIndex].column[5]
		------------ text update
		dataText = thisData["cmd"]
		columnFrame.text:SetText(dataText)
		
		------------------------ column 6: text and texture available
		columnFrame = scrollFrame.line[lineIndex].column[6]
		columnFrame.text:SetText("Edit")
	end
end

function SlashLib.getListProps() -- propriétés des frames list de macros user
	local scrollProps = { -- scroll frame properties (see _scroll.lua for full settings)
		["height"] = SlashLib.getVar("maxVisibleLines") * SlashLib.getVar("vLine"),
		["width"] = SlashLib.getVar("widthList") - SlashLib.getVar("margin"),
		["marginH"] = 1, -- px (margins from background frame : depends on template style)
		["marginV"] = 1, -- px (margins from background frame : depends on template style)
		["alwaysShowArrow"] = true, -- if true, always show scroller up and down arrows
		-------------------- lines
		["lineHeight"] = SlashLib.getVar("vLine"), -- px
		["lineSpacing"] = 1, -- px
		["cellSpacing"] = 0, -- px // horizontal spacing, if several columns in a line
		-------------------- lines font
		["font"] = SlashLib.getStuffPath("Myriad Condensed Web.ttf", true), -- "friz" | "arial" | pathToLocalFont accepted
		["fontSize"] = 12,
		["fontOutline"] = false,
		["fontColor"] = "EEEEEE", -- Hexacolor | RGB table | color name ("white" if NIL)
		["fontAlign"] = "LEFT", -- "LEFT" | "RIGHT" | "CENTER"
		["fontAlpha"] = 0.8,
		-------------------- lines background
		["background"] = {0.1, 0.1, 0.1},
		["border"] = 0, -- only available if ["background"] is not a texture path
		["borderColor"] = nil, -- only available if ["background"] is not a texture path
		["alpha"] = 0.4,
		["mouseOverLineColor"] = nil, -- visual color effect on mouseOver all the lines (nil for no mouseOver effect)
		-------------------- scroller buttons images
		["defaultScroller"] = false, -- set to true to standard WoW scroller
		["scrollerWidth"] = 16, -- px : unused if defaultScroller set to true
		["scrollerColor"] = "111122", -- unused if defaultScroller set to true
		["scrollerBarAlpha"] = 0.6, -- unused if defaultScroller set to true
	}
	
	local scrollColumns = { -- columns definition and properties (see _scroll.lua for full settings)
		-- column order, do not use name !
		[1] = { -- btns modif ordre UP
			---------- required
			["width"] = 10,
			["type"] = "Button", -- "Frame" (default), "Button" or "EditBox"
			---------- optional script (for button "OnClick" or EditBox "OnEnterPressed")
			["script"] = "SlashLib.listClickUserLine",
			---------- optional font properties (override scroller properties)
			["fontColor"] = {0.73,0.39,0.08}, -- Hexacolor | RGB table | color name (if nil : yellow)
			["fontSize"] = 11,
			["fontAlign"] = "CENTER",
			---------- optional background properties (override scroller properties)
			["alpha"] = 0.4,
			["mouseOverLineColor"] = {0.3, 0.3, 0.3}, -- visual color effect on mouseOver for this frame
		},
		[2] = { -- btns modif ordre DOWN
			---------- required
			["width"] = 10,
			["type"] = "Button", -- "Frame" (default), "Button" or "EditBox"
			---------- optional script (for button "OnClick" or EditBox "OnEnterPressed")
			["script"] = "SlashLib.listClickUserLine",
			---------- optional font properties (override scroller properties)
			["fontColor"] = {0.73,0.39,0.08}, -- Hexacolor | RGB table | color name (if nil : yellow)
			["fontSize"] = 11,
			["fontAlign"] = "CENTER",
			---------- optional background properties (override scroller properties)
			["alpha"] = 0.4,
			["mouseOverLineColor"] = {0.3, 0.3, 0.3}, -- visual color effect on mouseOver for this frame
		},
		[3] = { -- visible btn
			---------- required
			["width"] = 30,
			["type"] = "Button", -- "Frame" (default), "Button" or "EditBox"
			---------- optional script (for button "OnClick" or EditBox "OnEnterPressed")
			["script"] = "SlashLib.listClickUserLine", -- NIL for no script
			---------- optional font properties (override scroller properties)
			["fontColor"] = {0.73,0.39,0.08}, -- Hexacolor | RGB table | color name (if nil : yellow)
			["fontSize"] = 11,
			["fontAlign"] = "CENTER",
			---------- optional background properties (override scroller properties)
			["background"] = {0.15, 0.15, 0.15},
			["alpha"] = 0.7,
			["mouseOverLineColor"] = {0, 0.1, 0.1}, -- visual color effect on mouseOver for this frame
		},
		[4] = { -- name
			---------- required
			["width"] = 150, -- % value accepted. if 0 or NIL : all the remaining line's width (for last column only!)
			["type"] = "Button", -- "Frame" (default), "Button" or "EditBox"
			---------- optional script (for button "OnClick" or EditBox "OnEnterPressed")
			["script"] = "SlashLib.listClickUserLine", -- NIL for no script
			---------- optional font properties (override scroller properties)
			["fontAlpha"] = 0.8,
			---------- optional background properties (override scroller properties)
			["alpha"] = 0.7,
			["mouseOverLineColor"] = {0.3, 0.3, 0.3}, -- visual color effect on mouseOver for this frame
		},
		[5] = { -- cmd
			---------- required
			["width"] = 290, -- % value accepted. if 0 or NIL : all the remaining line's width (for last column only!)
			["type"] = "Frame", -- "Frame" (default), "Button" or "EditBox"
			---------- optional font properties (override scroller properties)
			["fontAlpha"] = 0.6,
			---------- optional background properties (override scroller properties)
			["background"] = {0.8, 0.8, 0.2}, -- color, or texture file path
			["alpha"] = nil,
		},
		[6] = { -- Edit btn
			---------- required
			["width"] = 0, -- % value accepted. if 0 or NIL : all the remaining line's width (for last column only!)
			["type"] = "Button", -- "Frame" (default), "Button" or "EditBox"
			---------- optional script (for button "OnClick" or EditBox "OnEnterPressed")
			["script"] = "SlashLib.listClickUserLine", -- NIL for no script
			---------- optional font properties (override scroller properties)
			["fontColor"] = {0.74,0.64,0.07}, -- Hexacolor | RGB table | color name (if nil : yellow)
			["fontSize"] = 11,
			["fontAlign"] = "CENTER",
			---------- optional background properties (override scroller properties)
			["background"] = {0.44,0.18,0.13},
			["alpha"] = 0.8,
			["mouseOverLineColor"] = {0.3, 0.2, 0.3}, -- visual color effect on mouseOver for this frame
		},
	}
	
	return scrollProps, scrollColumns
end

---------------------------------------------------------------------------------------
---------------------------- CLIC LIGNES LISTES USER CMDS -----------------------------

function SlashLib.listClickUserLine(scrollFrameName, clickedColumnName, mouse, lineIndex) -- do not modify args !
	-- mouse: "LeftButton" | "RightButton"
	-- if (IsModifierKeyDown()...)
	
	local scrollFrame = _G[scrollFrameName] -- main scroll frame
	local dataLineIndex, dataLine = SlashLib.scrollGetDataLine(scrollFrame, lineIndex)
if (SlashLib.debug()) then SlashLib.echoDebug(dataLine, "clic user line: dataLineIndex="..SlashLib.nilSiNul(dataLineIndex)) end --/////
	if (not SlashLib.isEmpty(dataLine)) then
		--------- visual effect
		local clickedFrame = _G[clickedColumnName] -- .text and .texture available
		SlashLib.flash(clickedFrame.texture, 0.2, 0)
		--------- action
		local colNum = SlashLib.strToNumber(string.sub(clickedColumnName, -1, -1)) -- last char
		if (colNum == 1) then --up
			SlashLib.moveUpDown(dataLineIndex, dataLine, true)
		elseif (colNum == 2) then --down
			SlashLib.moveUpDown(dataLineIndex, dataLine, false)
		elseif (colNum == 3) then --visible
			SlashLib.setVisible(dataLineIndex, dataLine)
		elseif (colNum == 4) then --exec
			SlashLib.execLine(dataLineIndex, dataLine)
		elseif (colNum == 6) then --edit
			SlashLib.editLine(dataLineIndex, dataLine)
		end
	else
		SlashLib.echoChatAppl("unexpected error #3700", false, "red")
	end
end

function SlashLib.moveUpDown(dataLineIndex, dataLine, upBoolean)
	--if (dataLine.perchar) then
end

function SlashLib.setVisible(dataLineIndex, dataLine) -- switch
	local name, state, color
	if (dataLine.perchar) then
		SLIB_SAVED_PERCHAR["lines"][dataLineIndex].visible = not SLIB_SAVED_PERCHAR["lines"][dataLineIndex].visible
		SlashLib.scrollUpdate(framePerCharList, SLIB_SAVED_PERCHAR["lines"])
		state = SLIB_SAVED_PERCHAR["lines"][dataLineIndex].visible
		name = SLIB_SAVED_PERCHAR["lines"][dataLineIndex].name
	else
		SLIB_SAVED_DATA["lines"][dataLineIndex].visible =  not SLIB_SAVED_DATA["lines"][dataLineIndex].visible
		SlashLib.scrollUpdate(frameGeneralList, SLIB_SAVED_DATA["lines"])
		state = SLIB_SAVED_DATA["lines"][dataLineIndex].visible
		name = SLIB_SAVED_DATA["lines"][dataLineIndex].name
	end
	if (state) then
		color = "aqua"
		state = SlashLib.getLocal("is now visible.")
	else
		color = "yellow"
		state = SlashLib.getLocal("is no longer visible.")
	end
	SlashLib.echoChatAppl(SlashLib.kwote(name).." "..state, false, color)
end

function SlashLib.execLine(dataLineIndex, dataLine)
	if (dataLine.perchar) then
		SlashLib.execCmd(SLIB_SAVED_PERCHAR["lines"][dataLineIndex].cmd, SLIB_SAVED_PERCHAR["lines"][dataLineIndex].kind)
	else
		SlashLib.execCmd(SLIB_SAVED_DATA["lines"][dataLineIndex].cmd, SLIB_SAVED_DATA["lines"][dataLineIndex].kind)
	end
end

function SlashLib.editLine(dataLineIndex, dataLine) -- chargement d'une ligne dans la zone d'édition
	SlashLib.fillEditMacro(dataLine, dataLineIndex)
	SlashLibZone_Edit_EditBox_cmd:SetFocus()
	SlashLibZone_Edit_EditBox_cmd:HighlightText(string.len(dataLine.cmd))
end

function SlashLib.fillEditMacro(tblMacro, numLiEdited)
	local numLiEdited = numLiEdited or 0
	local tblMacro = tblMacro or SlashLib.getNewMacro() -- pour btn "New"
	SlashLib.setEditData(SlashLib.getNewMacro(tblMacro.idtime, tblMacro.index, tblMacro.kind, tblMacro.perchar, tblMacro.name, tblMacro.cmd, tblMacro.visible), numLiEdited)
	SlashLibZone_Edit_EditBox_name:SetText(tblMacro.name)
	SlashLibZone_Edit_EditBox_cmd:SetText(tblMacro.cmd)
	SlashLibZone_Edit_Perchar.setChecked(tblMacro.perchar)
	SlashLib.updateSaveButton()
end

---------------------------------------------------------------------------------------
------------------------------ LISTE SLASH-CMDS DISPOS --------------------------------

function SlashLib.getAvailableSlashCmds() -- table de TOUTES les slash-cmds dispos
	local tbl = {}
	for key, value in pairs(hash_SlashCmdList) do -- in 4.3 SlashCmdList is copied to this table and wiped
		table.insert(tbl, string.lower(key))
	end
	table.sort(tbl)
	return tbl
end

function SlashLib.createMacrosFrame(parent, width, height)
	frameSlashList = CreateFrame("Frame", "SlashLibFrameSlashList", parent)
	SlashLib.setColoredFrame(frameSlashList, "112222", "222222", 1, 0.5)
	frameSlashList.allCmds = SlashLib.getAvailableSlashCmds() -- propriété
	local scrollProps = { -- scroll frame properties (see _scroll.lua for full settings)
		["height"] = height,
		["width"] = width - (SlashLib.getVar("margin") * 2),
		["marginH"] = 1, -- px (margins from background frame : depends on template style)
		["marginV"] = 1, -- px (margins from background frame : depends on template style)
		["alwaysShowArrow"] = true, -- if true, always show scroller up and down arrows
		-------------------- lines
		["lineHeight"] = SlashLib.getVar("vLine"), -- px
		["lineSpacing"] = 1, -- px
		["cellSpacing"] = 0, -- px // horizontal spacing, if several columns in a line
		-------------------- lines font
		["font"] = SlashLib.getStuffPath("Myriad Condensed Web.ttf", true), -- "friz" | "arial" | pathToLocalFont accepted
		["fontSize"] = 11,
		["fontOutline"] = false,
		["fontColor"] = "CCC033", -- Hexacolor | RGB table | color name ("white" if NIL)
		["fontAlign"] = "CENTER", -- "LEFT" | "RIGHT" | "CENTER"
		["fontAlpha"] = 0.8,
		-------------------- lines background
		["background"] = {0.08, 0.08, 0.08},
		["border"] = 0, -- only available if ["background"] is not a texture path
		["borderColor"] = "black", -- only available if ["background"] is not a texture path
		["alpha"] = 0.5,
		["mouseOverLineColor"] = "333333", -- visual color effect on mouseOver all the lines (nil for no mouseOver effect)
		-------------------- scroller buttons images
		["defaultScroller"] = false, -- set to true to standard WoW scroller
		["scrollerWidth"] = 16, -- px : unused if defaultScroller set to true
		["scrollerColor"] = "111122", -- unused if defaultScroller set to true
		["scrollerBarAlpha"] = 0.6, -- unused if defaultScroller set to true
	}
	
	local scrollColumns = { -- columns definition and properties (see _scroll.lua for full settings)
		-- column order, do not use name !
		[1] = {
			---------- required
			["width"] = nil, -- toute la largeur
			["type"] = "Button", -- "Frame" (default), "Button" or "EditBox"
			---------- optional script (for button "OnClick" or EditBox "OnEnterPressed")
			["script"] = "SlashLib.macrosClickLineScript_run",
		},
	}
	
	SlashLib.scrollCreate(frameSlashList, scrollProps, scrollColumns, frameSlashList.allCmds)
end

function SlashLib.macrosClickLineScript_run(scrollFrameName, clickedColumnName, mouse, lineIndex) -- do not modify args !
	-- mouse: "LeftButton" | "RightButton"
	-- if (IsModifierKeyDown()...)
	local modifiedClick = (mouse == "RightButton") or IsModifierKeyDown()
	local scrollFrame = _G[scrollFrameName] -- scroll frame
	local clickedFrame = _G[clickedColumnName] -- .text and .texture available
	
	local dataLineIndex, dataLine = SlashLib.scrollGetDataLine(scrollFrame, lineIndex)
	if (not SlashLib.isEmpty(dataLine)) then
		SlashLib.flash(clickedFrame.texture, 0.2, 0)
		if (modifiedClick) then
			SlashLibZone_Edit_EditBox_cmd:SetText(SlashLibZone_Edit_EditBox_cmd:GetText()..dataLine)
			SlashLibZone_Edit_EditBox_cmd:SetFocus()
			SlashLibZone_Edit_EditBox_cmd:HighlightText(string.len(SlashLibZone_Edit_EditBox_cmd:GetText()))
		else
			SlashLib.executeCmd(dataLine, true) -- lib
		end
	else
		SlashLib.echoChatAppl("unexpected error #3800", false, "red")
	end
end
